Attribute VB_Name = "modMain"
Option Explicit

Private m_FileObject As FILECTLCtl.File

Public Function GetFileObject() As FILECTLCtl.File
    On Error Resume Next
    
    ' If we haven't created the File object yet, do so
    If IsEmpty(m_FileObject) Then
        ' Create a File control
        Set m_FileObject = CreateObject("FILECTL.File")
        
        ' Ensure the object was created successfully
        If Err.Number <> 0 Then
            MsgBox "Failed to create File object." & vbCrLf & _
                "Ensure MSCEFile.dll has been installed and registered.", _
                vbCritical, "Error"
            Exit Function
        End If
    End If
    
    ' Return our global File object
    Set GetFileObject = m_FileObject
End Function

Public Sub ReleaseFileObject()
    Set m_FileObject = Nothing
End Sub

Public Sub ReadDirectory(Path As String, _
                        fsFileSystem As FILECTLCtl.FileSystem, _
                        tvwTreeView As MSCETREEVIEWLibCtl.TreeViewCtl, _
                        nodParentNode As MSCETREEVIEWLibCtl.Node, _
                        lstListBox As ListBox, _
                        strFilter As String)
                        
    Dim strDir As String
    Dim strFile As String
    
    On Error Resume Next
    
    ' Ensure path is slash terminated
    If Right(Path, 1) <> "\" Then Path = Path & "\"
    
    ' ----- First get all directories -----
    ' Priming Read
    strDir = fsFileSystem.Dir(Path & "*", fsAttrDirectory)
    
    ' Iterate through directory
    Do While strDir <> ""
        ' add directories to TreeView
        tvwTreeView.Nodes.Add nodParentNode.Key, tvwChild, strDir, strDir
        
        ' if the node already exists, ignore the error
        If Err.Number = 35602 Then
            ' ignore duplicate node adds - the add will fail anyway
        ElseIf Err.Number <> 0 Then
            MsgBox Err.Description, vbExclamation, "Error adding node!"
        End If
        
        strDir = fsFileSystem.Dir
    Loop
    
    ' ----- Next get all files -----
    ' Clear ListBox
    lstListBox.Clear
    
    ' Priming Read
    strFile = fsFileSystem.Dir(Path & strFilter, fsAttrNormal)
    
    ' Iterate through directory
    Do While strFile <> ""
        'Add Files to ListBox
        lstListBox.AddItem strFile
        strFile = fsFileSystem.Dir
    Loop
End Sub

Public Function GetFileTextContents(Path As String) As String
    Dim filFile As FILECTLCtl.File
    Dim strInput As String
    
    On Error Resume Next
    
    ' Get our application File object
    Set filFile = GetFileObject()
    
    ' Open the File
    filFile.Open Path, fsModeInput, fsAccessRead
    
    ' Make sure the call to Open was successful
    If Err.Number <> 0 Then
        MsgBox "Open Method failed." & vbCrLf & _
            "The file could not be read.", _
            vbCritical, "Error"
    End If
    
    ' Loop through file, filling our input buffer
    Do While Not filFile.EOF
        strInput = strInput & filFile.Input(1)
    Loop
    
    ' Close the file
    filFile.Close
    
    ' Release the File Object
    Set filFile = Nothing
    
    ' Return the text
    GetFileTextContents = strInput
End Function

Public Sub CreateFile(Path As String, Contents As String)
    ' This function performs double duty. We use it to create
    '   new files as well as overwrite old files
    
    Dim filFile As FILECTLCtl.File
    
    On Error Resume Next
    
    ' Get our application File object
    Set filFile = GetFileObject()
    
    ' Open the file, if it doesn't exist, it will be created
    ' fsModeOutput means overwrite
    filFile.Open Path, fsModeOutput, fsAccessWrite, fsLockWrite
           
    ' Make sure the call to Open was successful
    If Err.Number <> 0 Then
        MsgBox "Open Method failed." & vbCrLf & _
            "The file could not be created.", _
            vbCritical, "Error"
    End If
    
    ' Write our data to the file if it's not empty
    If Contents <> "" Then
        filFile.LinePrint Contents
    End If
    
    ' Close the file
    filFile.Close
    
    ' Release the File Object
    Set filFile = Nothing
End Sub

Public Function GetFileAttributes(Path As String, _
                                    fsFileSystem As FILECTLCtl.FileSystem) _
                                    As String
                                    
    Dim FileAttr As FileAttrEnum
    Dim lFileLength As Long
    Dim dtFileDate As Date
    Dim strAttributes As String
    Dim strFileName As String
    
    On Error Resume Next
    
    ' Get the attributes.  A file can have many
    FileAttr = fsFileSystem.GetAttr(Path)
    
    ' Check if it's a system file
    If FileAttr And fsAttrSystem Then
        strAttributes = strAttributes & "System File" & vbCrLf
    End If
    If FileAttr And fsAttrReadOnly Then
        strAttributes = strAttributes & "Read-Only" & vbCrLf
    End If
    If FileAttr And fsAttrArchive Then
        strAttributes = strAttributes & "Archive File" & vbCrLf
    End If
    
    ' Get file's date
    dtFileDate = fsFileSystem.FileDateTime(Path)
    strAttributes = strAttributes & dtFileDate & vbCrLf

    ' Get file's length
    lFileLength = fsFileSystem.FileLen(Path)
    
    ' format the size a little nicer
    ' Since Format() isn't supported, we'll use integer/float division
    '   as a workaround.
    If lFileLength < 1024 Then
        strAttributes = strAttributes & CStr(lFileLength) & " bytes"
    ElseIf lFileLength < 1048576 Then
        ' Go out 1 decimal place
        strAttributes = strAttributes & CStr((lFileLength \ 102.4) / 10) & "k bytes"
    Else
        ' Go out 2 decimal places
        strAttributes = strAttributes & CStr((lFileLength \ 10485.76) / 100) & "M bytes"
    End If
    
    ' Return the attributes
    GetFileAttributes = strAttributes
End Function

