Attribute VB_Name = "modGlobal"
Option Explicit

' constants for mode that the
' cd form can operate in
Const CDFORM_MODE_EDIT = 0
Const CDFORM_MODE_NEW = 1

' contstants for the ADOCE classes
Const ADOCE_CLASS_CONNECTION = "ADOCE.Connection.3.0"
Const ADOCE_CLASS_RECORDSET = "ADOCE.Recordset.3.0"

' contstant for database path and name
Const CD_CDB_PATH_FILENAME = "\cd.cdb"

' global objects for all ADOCE objects
Public gobjConnection As ADOCE.Connection
Public gobjArtist As ADOCE.Recordset
Public gobjCD As ADOCE.Recordset
Public gobjPKEY As ADOCE.Recordset
Public gobjGenre As ADOCE.Recordset

Function InitializeGlobals() As Boolean

    On Error Resume Next
    
    InitializeGlobals = False
    
    ' initialize globals - this needs to take place
    ' only once, due to CreateObject memory leak
    
    ' ADOCE_CLASS_CONNECTION & ADOCE_CLASS_RECORDSET
    ' are used because ce references these classes
    ' by their version numbers, create Consts allows
    ' changing of these version quickly and accurately
    
    Set gobjConnection = CreateObject(ADOCE_CLASS_CONNECTION)
    Set gobjArtist = CreateObject(ADOCE_CLASS_RECORDSET)
    Set gobjCD = CreateObject(ADOCE_CLASS_RECORDSET)
    Set gobjGenre = CreateObject(ADOCE_CLASS_RECORDSET)
    Set gobjPKEY = CreateObject(ADOCE_CLASS_RECORDSET)
    
    If Err.Number = 0 Then
        InitializeGlobals = True
    End If

End Function

Function ReleaseGlobals() As Boolean

    On Error Resume Next
    
    ReleaseGlobals = True
    
    'release all global object
    Set gobjConnection = Nothing
    Set gobjArtist = Nothing
    Set gobjCD = Nothing
    Set gobjPKEY = Nothing
    Set gobjGenre = Nothing
    
    If Err.Number = 0 Then
        ReleaseGlobals = True
    End If

End Function
Function OpenDatabase() As Boolean

    On Error Resume Next
    
    ' set the return value to an error
    ' condition, so we can exit early
    ' if an error occurs
    OpenDatabase = False
    
    ' make sure the database exists
    If Not EnsureDatabaseExists() Then
        Exit Function
    End If
    
    ' we need to check if the connection is
    ' open or close, because EnsureDatabaseExists may
    ' opened the database
    
    If gobjConnection.State = adStateClosed Then
        ' opens the pocket access database
        gobjConnection.Open CD_CDB_PATH_FILENAME
    End If
    
    If Err.Number = 0 Then
        OpenDatabase = True
    End If

End Function

Function EnsureDatabaseExists() As Boolean
    
    Dim strDir As String
    
    On Error Resume Next
    
    ' set the return value to an error
    ' condition, so we can exit early
    ' if an error occuers
    EnsureDatabaseExists = False
    
    ' check to see if the database exists
    strDir = frmMain.FileSystem1.Dir(CD_CDB_PATH_FILENAME)
    
    ' if it doesn't let's create it
    If Len(strDir) = 0 Then
        ' prepares the connection for use
        gobjConnection.Open
        
        ' creates a pocket access database
        gobjConnection.Execute "CREATE DATABASE '" & CD_CDB_PATH_FILENAME & "'"
        
        ' closes the connection - can not open
        ' the database while the connection is open
        gobjConnection.Close
        
        ' reopen the database (using the cd.cdb)
        If Not OpenDatabase() Then
            MsgBox "Can not open database."
            Exit Function
        End If
        
        ' create the tables and indexes
        If Not CreateTables() Then
            MsgBox "Can not create tables."
            Exit Function
        End If
    
        ' create test data
        If Not CreateData() Then
            MsgBox "Error occurred in creating sample data."
            Exit Function
        End If
    
    End If
    
    
    ' if no errors have occurred, we
    ' have successfully completed this
    ' function, so let's return a
    ' successful return value
    If Err.Number = 0 Then
        EnsureDatabaseExists = True
    End If

End Function


Function CreateTables() As Boolean

    On Error Resume Next
    
    CreateTables = False
    
    ' creates the table Artist
    gobjConnection.Execute "CREATE TABLE Artist " _
        & "(ArtistId int, ArtistName varchar(64), DefaultGenre int)"
    ' creates the table CD
    gobjConnection.Execute "CREATE TABLE CD " _
        & "(CDId int, ArtistId int, CDName varchar(64), " _
        & "Year int, Price float, GenreId int)"
    ' creates the Genre table
    gobjConnection.Execute "CREATE TABLE Genre " _
        & "(GenreID int, GenreName varchar(64))"
    ' creates the PKey table (primary key)
    gobjConnection.Execute "CREATE TABLE PKey " _
        & "(TableName varchar(20), KeyValue int)"
    
    ' creates an index on ArtistId on table CD
    gobjConnection.Execute "CREATE INDEX ArtistId ON CD (ArtistId)"
    
    ' creates an index on TableName on table PKey
    gobjConnection.Execute "CREATE INDEX TableName ON PKey (TableName)"
    
    ' creates an index on GenreId on table Genre
    gobjConnection.Execute "CREATE INDEX Genre ON Genre (GenreId)"
    
    ' populates the PKey table with a record for the Artist table
    gobjConnection.Execute "INSERT INTO PKey VALUES (""Artist"", 0 )"
    ' populates the PKey table with a record for the CD table
    gobjConnection.Execute "INSERT INTO PKey VALUES (""CD"", 0 )"
    ' populates the PKey table with a record for the Genre table
    gobjConnection.Execute "INSERT INTO PKey VALUES (""Genre"", 0 )"
    
    If Err.Number = 0 Then
        CreateTables = True
    End If

End Function

Function GetNewPK(ByVal strTable As String) As Long

    ' gets a new pk for a particular table
    
    On Error Resume Next
    
    ' set the return value in case we exit early
    GetNewPK = -1
    
    ' if the table is open, let's close it
    If gobjPKEY.State = adStateOpen Then
        gobjPKEY.Close
    End If
    
    ' open the table with the correct record
    gobjPKEY.Open "SELECT KeyValue FROM PKey WHERE TableName = " _
        & """" & strTable & """", gobjConnection, _
        adOpenForwardOnly, adLockOptimistic
    
    ' if there is no record for this table
    ' we have an error
    If gobjPKEY.EOF Then
        Exit Function
    End If
    
    ' set the value to itself plus one
    gobjPKEY.Fields("KeyValue").Value = gobjPKEY.Fields("KeyValue").Value + 1
    
    ' update the record
    gobjPKEY.Update
    
    ' set the return value
    GetNewPK = gobjPKEY.Fields("KeyValue").Value
    
    ' if we had an error, let's set the
    ' return value to a negative return
    If Err.Number <> 0 Then
        GetNewPK = -1 * Abs(Err.Number)
    End If

End Function

Function AddGenre(ByVal strGenre As String) As Long

    ' adds an genre to the table and returns
    ' its primary key value

    Dim lngGenreId As Long
    
    On Error Resume Next
    
    ' set the return value to error code
    ' in case we need to exit early
    AddGenre = -1
    
    ' get a new primary key value
    lngGenreId = GetNewPK("Genre")
    
    ' check the return value for the new primary key
    If lngGenreId < 0 Then
        Exit Function
    End If
    
    ' if the recordset is open
    ' let's check if it supports addnew
    If gobjGenre.State = adStateOpen Then
        ' if the current recordset does not
        ' support adding new records, let's close
        ' it and reopen it
        If Not gobjGenre.Supports(adAddNew) Then
            gobjGenre.Close
        End If
    End If
    
    ' if the recordset is closed above
    ' or closed before we get into this
    ' function, let's open it with the
    ' proper support for adding
    If gobjGenre.State = adStateClosed Then
        gobjGenre.Open "genre", gobjConnection, _
            adOpenForwardOnly, adLockOptimistic, adCmdTableDirect
    End If
    
    ' add a new record
    gobjGenre.AddNew
    
    ' set the values
    gobjGenre.Fields("GenreId").Value = lngGenreId
    gobjGenre.Fields("GenreName").Value = strGenre
    
    ' update the recordset
    gobjGenre.Update

    ' set the return value
    AddGenre = lngGenreId
    
    ' if there was an error, let's set the return
    ' value to a negative value
    If Err.Number <> 0 Then
        AddGenre = -1 * Abs(Err.Number)
    End If
    
End Function

Function AddArtist(ByVal strArtist As String, ByVal lngDefaultId As Long) As Long

    ' adds an artist to the table and returns
    ' its primary key value
    ' if there is an error, returns a negative value
    
    Dim lngArtistId As Long
    
    On Error Resume Next
    
    ' set the return value to an error value
    ' in case we exit early
    AddArtist = -1
    
    
    ' returns a new id for the new artist
    lngArtistId = GetNewPK("Artist")
    
    ' checks the return value
    If lngArtistId < 0 Then
        Exit Function
    End If
    
        
    ' needs to test the recordset to see if
    ' it currently supports adding new records
    
    ' let's see if recordset is open
    If gobjArtist.State = adStateOpen Then
        ' if it currently does not support
        ' adding new, let's close it and reopen
        ' it below
        If Not gobjArtist.Supports(adAddNew) Then
            ' close the recordset
            gobjArtist.Close
        End If
    End If
    
    ' if it was closed before we got into this
    ' function or closed in this function, let's
    ' reopen it with the correct support (AddsNew)
    If gobjArtist.State = adStateClosed Then
        gobjArtist.Open _
            "artist", _
            gobjConnection, _
            adOpenForwardOnly, _
            adLockOptimistic, _
            adCmdTableDirect
        ' "artist" - open the table direct
        ' gobjConnection - the global connection object
        ' adOpenForwardOnly - we're only going to add,
        '   so no need to have scrollable cursor
        ' adLockOptimistic - quick updates
        ' adCmdTableDirect - optional parameter that
        ' informs the recordset to open the table directly
        ' allowing the quickest manipulation
    End If
    
    
    ' append a new record
    gobjArtist.AddNew
    
    ' set values
    gobjArtist.Fields("ArtistId").Value = lngArtistId
    gobjArtist.Fields("ArtistName").Value = strArtist
    
    ' if we have a genre, let's set it
    If Not IsEmpty(lngDefaultId) Then
        gobjArtist.Fields("DefaultGenre").Value = lngDefaultId
    End If
    
    ' update the recordset - commits the record
    gobjArtist.Update
    
    ' set the return value
    AddArtist = lngArtistId
    
    ' if we had an error, let's set the
    ' return value to a negative value
    ' so we know that it failed
    If Err.Number <> 0 Then
        AddArtist = -1 * Abs(Err.Number)
    End If

End Function

Function AddCD( _
    ByVal lngArtistId As Long, _
    ByVal lngGenreId As Long, _
    ByVal strCDName As String, _
    ByVal lngYear As Long, _
    ByVal curPrice As Currency) As Long
    
    Dim lngCDId As Long
    
    On Error Resume Next
    
    ' let's set the return value to
    ' an error code, so if we exit early
    AddCD = -1
    
    ' get a new primary key value
    lngCDId = GetNewPK("CD")
    
    ' check the value to make sure it valid
    If lngCDId < 0 Then
        Exit Function
    End If
    
    ' if the CD recordset is open, let's
    ' check to make sure it can support
    ' adding a new record
    If gobjCD.State = adStateOpen Then
        ' check to see if the current
        ' recordset supports adding a new record
        If Not gobjCD.Supports(adAddNew) Then
            gobjCD.Close
        End If
    End If
    
    ' if the recordset is closed, either
    ' before we entered the function
    ' or if we closed it above
    If gobjCD.State = adStateClosed Then
        gobjCD.Open "cd", gobjConnection, adOpenForwardOnly, _
            adLockOptimistic, adCmdTableDirect
    End If

    
    ' add a new record
    gobjCD.AddNew
    ' set the values
    gobjCD.Fields("CDId").Value = lngCDId
    
    ' let's set the Artist value
    gobjCD.Fields("ArtistId").Value = lngArtistId
    
    ' let's set the Genre value
    gobjCD.Fields("GenreId").Value = lngGenreId
    
    ' if we have an cd name, let's set the value
    If Not IsEmpty(strCDName) Then
       gobjCD.Fields("CDName").Value = strCDName
    End If
    
    ' if we have an year, let's set the value
    If Not IsEmpty(lngYear) Then
       gobjCD.Fields("Year").Value = lngYear
    End If
    
    ' if we have an price, let's set the value
    If Not IsEmpty(curPrice) Then
       gobjCD.Fields("Price").Value = curPrice
    End If
    
    ' update the recordset
    gobjCD.Update

    ' set the return value
    AddCD = lngCDId
    
    ' if we had an error, let's set
    ' the return value to a negative value
    If Err.Number <> 0 Then
        AddCD = -1 * Abs(Err.Number)
    End If
    

End Function
Private Function CreateData() As Boolean

    Dim lngRockId As Long
    Dim lngBlueGrassId As Long
    Dim lngHeavyMetalId As Long
    
    Dim lngTomPettyId As Long
    Dim lngAllisonKrausId As Long
    Dim lngMetallicaId As Long
    
    On Error Resume Next
    
    ' set the return value to an
    ' error condition, this allows
    ' early exit if an error occurs
    CreateData = False
    
    ' let's add genres
    lngRockId = AddGenre("Rock")
    
    ' check the return value, should be
    ' greater than zero
    If lngRockId < 0 Then
        Exit Function
    End If
    
    lngBlueGrassId = AddGenre("Bluegrass")
    ' check the return value, should be
    ' greater than zero
    If lngBlueGrassId < 0 Then
        Exit Function
    End If
    
    lngHeavyMetalId = AddGenre("Heavy Metal")
    ' check the return value, should be
    ' greater than zero
    If lngHeavyMetalId < 0 Then
        Exit Function
    End If

    ' let's add some artists
    lngTomPettyId = AddArtist("Tom Petty", lngRockId)
    ' check the return value, should be
    ' greater than zero
    If lngTomPettyId < 0 Then
        Exit Function
    End If
    
    lngAllisonKrausId = AddArtist("Allison Kraus", lngBlueGrassId)
    ' check the return value, should be
    ' greater than zero
    If lngAllisonKrausId < 0 Then
        Exit Function
    End If
    
    lngMetallicaId = AddArtist("Metallica", lngHeavyMetalId)
    ' check the return value, should be
    ' greater than zero
    If lngMetallicaId < 0 Then
        Exit Function
    End If
    
    ' add cds for Tom Petty
    If AddCD(lngTomPettyId, lngRockId, _
        "Damn the Torpedos", 1989, 10.99) < 0 Then
            Exit Function
    End If
    
    If AddCD(lngTomPettyId, lngRockId, _
        "Tom Petty Live", 1988, 12.99) < 0 Then
            Exit Function
    End If

    ' add cds for Allison Kraus
    If AddCD(lngAllisonKrausId, lngBlueGrassId, _
        "Best Of Allison Kraus", 1998, 12.99) < 0 Then
            Exit Function
    End If
    
    If AddCD(lngAllisonKrausId, lngBlueGrassId, _
        "Long and Blue", 1999, 15.99) < 0 Then
            Exit Function
    End If

    ' add cds for Metallica
    If AddCD(lngMetallicaId, lngHeavyMetalId, _
        "Master Of The Puppets", 1986, 9.99) < 0 Then
            Exit Function
    End If
    
    If AddCD(lngMetallicaId, lngHeavyMetalId, _
        "Metallica", 1983, 12.99) < 0 Then
            Exit Function
    End If
    
    If Err.Number = 0 Then
        CreateData = True
    End If

End Function


Function RecreateDatabase() As Boolean
    
    On Error Resume Next
    
    ' set to an error condition
    RecreateDatabase = False
    
    If Not DeleteDataBase() Then
        Exit Function
    End If
    
    If Not InitializeGlobals() Then
        Exit Function
    End If
    
    If Not EnsureDatabaseExists() Then
        Exit Function
    End If
    
    OpenDatabase
End Function



Private Function DeleteDataBase() As Boolean

    On Error Resume Next
    
    ' set to an error condition
    DeleteDataBase = False
    
    If gobjArtist.State = adStateOpen Then
        gobjArtist.Close
    End If
    
    Set gobjArtist.ActiveConnection = Nothing
    
    If gobjCD.State = adStateOpen Then
        gobjCD.Close
    End If
    
    Set gobjCD.ActiveConnection = Nothing
    
    If gobjPKEY.State = adStateOpen Then
        gobjPKEY.Close
    End If
    
    Set gobjPKEY.ActiveConnection = Nothing
    
    If gobjGenre.State = adStateOpen Then
        gobjGenre.Close
    End If
    
    Set gobjGenre.ActiveConnection = Nothing
    
    ' check to see if the connection is open
    If gobjConnection.State = adStateOpen Then
        gobjConnection.Close
    End If
    
    'release all objects
    Set gobjArtist = Nothing
    Set gobjCD = Nothing
    Set gobjPKEY = Nothing
    Set gobjGenre = Nothing
    
    ' check to see if the connection is open
    If gobjConnection.State = adStateOpen Then
        gobjConnection.Close
    End If
    
    Set gobjConnection = Nothing
    
    ' delete the database
    frmMain.FileSystem1.Kill (CD_CDB_PATH_FILENAME)
    
    If Err.Number = 0 Then
        DeleteDataBase = True
    End If

End Function

