Attribute VB_Name = "modSQLRepl"
Option Explicit

' the database path and name
Public Const DB_PATH = "\CD.sdf"
' the config parms table
Public Const CONFIG_PARAMS_TABLE = "ConfigParms"

' class names
Public Const SSCE_REPLICATION_CLASS = "SSCE.Replication.1.0"
Public Const ADOXCE_CATALOG_CLASS = "ADOXCE.Catalog.3.1"

' AddSubscription Consts
Public Const EXISTING_DATABASE = 0
Public Const CREATE_DATABASE = 1

' NetworkType Consts
Public Const DEFAULT_NETWORK = 0
Public Const TCPIP_SOCKETS = 1
Public Const MULTI_PROTOCOL = 2

' Authentication Consts
Public Const DB_AUTHENTICATION = 0
Public Const NT_AUTHENTICATION = 1


Public gstrInternetURL As String
Public gstrPublisherAddress As String
Public gstrPublisherNetwork As SSCE.REPL_NETWORK_TYPE
Public gstrPublisher  As String
Public gPublisherSecurityMode As SSCE.REPL_SECURITY_TYPE
Public gstrPublisherLogin As String
Public gstrPublisherPassword As String
Public gstrSubscriberConnectionString  As String
Public gstrSubscriber  As String
Public gstrHostName  As String
Public gstrPublication As String
Public gstrPublisherDatabase As String
Public gstrIPAddress As String
Public gblnFirstStartUp As Boolean
Public glngCurrentUser As Long

' configuration recordset
Public gConfigRecord As ADOCE.Recordset
' ADOXCE Catalog - Used to check for the
' config table's existence
Public gobjADOXCatalog As ADOXCE.Catalog
' used to retrieve user id
Public gobjUser As ADOCE.Recordset
' the SQL Server CE replication object
Public gSQLReplication As SSCE.Replication


Sub Main()

    On Error Resume Next
    
    ' initialize all global variables
    If Not InitializeGlobals() Then
        MsgBox "Error initializing global objects."
        Exit Sub
    End If
    
    ' default to first startup
    gblnFirstStartUp = True
    
    ' if the database exists then set first start up to false
    If Len(frmSQLProperties.FileSystem1.Dir(DB_PATH)) > 0 Then
        gblnFirstStartUp = False
    End If
    
    ' if we can't load the configs, then inform the
    ' user and exit
    If Not LoadConfigs() Then
        MsgBox "Error loading configuration."
    End If
    
    ' if first startup
    If gblnFirstStartUp Then
        ' let's show the sql form
        ' allow user to check/set the values
        frmSQLProperties.Show
    Else
        ' we're all set, show the main form
        frmMain.Show
    End If
    
End Sub

Function Sync() As Boolean

    On Error Resume Next
    
    'set the error state so we can leave quickly
    Sync = False
    
    ' we have to close the database
    ' before we Sync to the remote server
    If Not CloseDatabase() Then
        ' problem closing - let's exit with
        ' an error condition
        Exit Function
    End If
    
    ' the URL for the SSCE dll running on IIS
    gSQLReplication.InternetURL = gstrInternetURL
    
    ' the SQL server network library setting
    gSQLReplication.PublisherNetwork = gstrPublisherNetwork
    
    ' name of the Publisher for replication
    gSQLReplication.Publisher = gstrPublisher
    
    ' name of the database
    gSQLReplication.PublisherDatabase = gstrPublisherDatabase
    
    ' name of the publication
    gSQLReplication.Publication = gstrPublication
    
    ' security mode to authenticate to the SQL server
    gSQLReplication.PublisherSecurityMode = gPublisherSecurityMode
    
    ' login
    gSQLReplication.PublisherLogin = gstrPublisherLogin
    
    ' password
    gSQLReplication.PublisherPassword = gstrPublisherPassword
    
    ' IP address of the publisher
    gSQLReplication.PublisherAddress = gstrIPAddress
    
    ' Set Subscriber properties
    gSQLReplication.SubscriberConnectionString = "Data Source=" & DB_PATH
    
    ' set the subscriber name
    gSQLReplication.Subscriber = gstrSubscriber
    
    ' set the host name
    gSQLReplication.HostName = gstrSubscriber
    
    ' if the first time, we need to create the DB
    ' while we're adding a subscription
    If gblnFirstStartUp Then
        gSQLReplication.AddSubscription CREATE_DATABASE
    End If
    
    ' initialize the subscription on the device
    gSQLReplication.Initialize
    
    ' check for errors
    If gSQLReplication.ErrorRecords.Count > 0 Then
        ' display any errors
        ConnErrRoutine gSQLReplication
    Else
        ' Sync with server
        On Error Resume Next
        gSQLReplication.Run
        On Error GoTo 0
        ' check for errors
        If gSQLReplication.ErrorRecords.Count > 0 Then
            ' display errors
            ConnErrRoutine gSQLReplication
        End If
    End If
    
    ' close out and complete
    gSQLReplication.Terminate
    
    ' open or reopen the database
    If Not OpenDatabase() Then
        Exit Function
    End If
    
    ' set the return value to true if no error
    Sync = (Err.Number = 0)
    
    ' check the status
    If Sync Then
        MsgBox "Sync completed successfully.", , "CD Library"
    Else
        MsgBox "Sync completed with errors.", , "CD Library"
    End If
    
End Function

Function SetUserId() As Boolean

    On Error Resume Next
    
    ' set to error condition
    SetUserId = False

    ' close the record set
    If gobjUser.State = adStateClosed Then
        gobjUser.Open "Users", gobjConnection, _
            adOpenForwardOnly, adLockReadOnly, adCmdTableDirect
    End If
    
    ' if end of file, there's a problem
    If gobjUser.EOF Then
        Exit Function
    End If
    
    ' set the current user - used to populate UserId in cd table
    glngCurrentUser = CLng(gobjUser.Fields("UserId").Value)
    
    ' set the return value
    SetUserId = Err.Number = 0
    

End Function

Public Sub ConnErrRoutine(ConnectionType As Object)
  
  Dim sTemp As String
  Dim strErr As String
  Dim objError As SSCE.SSCEError
  Dim objParam As SSCE.SSCEParam
  
  On Error Resume Next
  
  ' loop thru each error
  For Each objError In ConnectionType.ErrorRecords
    ' add the source to the msg string
    strErr = strErr & "Source: " & objError.Source & vbCrLf
    ' add the error number to the msg string
    strErr = strErr & "Error number: " & Trim(Hex(objError.Number)) & vbCrLf
    ' add the native error number (SQL CE) to the msg string
    strErr = strErr & "Native Number: " & Trim(objError.NativeError) & vbCrLf
    ' add the description to the msg string
    strErr = strErr & "Error Description: " & objError.Description & vbCrLf
    ' add the friendly error to the msg string
    strErr = strErr & SSCEErrorDescription(objError.NativeError, sTemp) & vbCrLf
    strErr = strErr & sTemp & vbCrLf
    
    ' are there any parameters to the error
    If objError.Params.Count > 0 Then
        ' go thru each parameter
        For Each objParam In objError.Params
            ' add each of the param to the string
            strErr = strErr & "Param=" & objParam.Param & vbCrLf
        Next
    End If
    ' show the msg
    MsgBox strErr, vbCritical
    ' clear the msg str
    strErr = ""
  Next
  
  
End Sub

Function SaveConfiguration() As Boolean

    
    Dim objADOXTable As ADOXCE.Table
    Dim blnTableFound As Boolean
    
    On Error Resume Next
    
    ' set error condition so we can exit early
    SaveConfiguration = False
    
    ' set the ADOX catalog connection to the db connection
    Set gobjADOXCatalog.ActiveConnection = gobjConnection
    
    ' set flag that we have not found
    ' the configuration table
    blnTableFound = False
    
    ' cycle through each table in the DB
    For Each objADOXTable In gobjADOXCatalog.Tables
        ' check if it is the ConfigParams Table
        If objADOXTable.Name = CONFIG_PARAMS_TABLE Then
            ' if we found it, set the flag and get out
            blnTableFound = True
            Exit For
        End If
    Next
    
    ' if we did not find the table, let's create it
    If Not blnTableFound Then
        ' create the table
        gobjConnection.Execute "CREATE TABLE " _
            & CONFIG_PARAMS_TABLE _
            & " ( InternetURL nvarchar(255), PublisherNetwork tinyint, " _
            & "Publisher nvarchar(255), PublisherDB nvarchar(255), " _
            & "PublisherSecurityMode tinyint, PublisherLogin nvarchar(255), " _
            & "PublisherPassword nvarchar(255), Subscriber nvarchar(255), " _
            & "HostName nvarchar(255), Publication nvarchar(255), " _
            & "IPAddress nvarchar(255))"
    End If
    
    ' open it for adding/updating
    gConfigRecord.Open CONFIG_PARAMS_TABLE, gobjConnection, _
        adOpenForwardOnly, adLockOptimistic, adCmdTableDirect
    
    ' there should only be one record, so if there
    ' are not, add a record
    If gConfigRecord.EOF Then
        ' add the record
        gConfigRecord.AddNew
    End If
    
    ' save all the configuration data
    gConfigRecord.Fields("InternetURL") = gstrInternetURL
    gConfigRecord.Fields("PublisherNetwork") = gstrPublisherNetwork
    gConfigRecord.Fields("Publisher") = gstrPublisher
    gConfigRecord.Fields("PublisherSecurityMode") = gPublisherSecurityMode
    gConfigRecord.Fields("PublisherLogin") = gstrPublisherLogin
    gConfigRecord.Fields("PublisherPassword") = gstrPublisherPassword
    gConfigRecord.Fields("Subscriber") = gstrSubscriber
    gConfigRecord.Fields("HostName") = gstrHostName
    gConfigRecord.Fields("Publication") = gstrPublication
    gConfigRecord.Fields("IPAddress") = gstrIPAddress
    gConfigRecord.Fields("PublisherDB") = gstrPublisherDatabase
    
    ' send the changes
    gConfigRecord.Update
    
    
    ' if we're done with no errors let's
    ' clear the first time flag and set
    ' return value to success
    If Err.Number = 0 Then
        gblnFirstStartUp = False
        SaveConfiguration = True
    End If

End Function

Public Function LoadConfigs() As Boolean

    On Error Resume Next
    
    ' set to error condition for early exit
    LoadConfigs = False
    
    ' are we here the first time
    ' if so, let's set the values
    If gblnFirstStartUp Then
        
        ' ip address of the web server
        gstrIPAddress = "192.0.0.1"
        
        ' URL for the SSCE dll
        gstrInternetURL = "http://" _
            & gstrIPAddress & "/ssce/sscesa10.dll"
        
        ' name of the Publisher (SQL)
        gstrPublisher = "tblap"
        
        ' security mode that the publisher uses
        gPublisherSecurityMode = DB_AUTHENTICATION
        
        ' login id for the publisher
        gstrPublisherLogin = "sa"
        
        ' password
        gstrPublisherPassword = ""
        
        ' network library for the publisher
        gstrPublisherNetwork = TCPIP_SOCKETS
        
        ' name of the database for the publication (on the publisher)
        gstrPublisherDatabase = "CD"
        
        ' name of the subscriber
        gstrSubscriber = "mobileuser1"
        
        ' host name - used to filter the data
        gstrHostName = "mobileuser1"
        
        ' name of the publication
        gstrPublication = "CDPublication"
    Else
    
        ' open the database to get the SQL CE Replication configurations
        If Not OpenDatabase() Then
            ' error - get out
            Exit Function
        End If
        
        
        ' open the config table
        gConfigRecord.Open CONFIG_PARAMS_TABLE, gobjConnection, _
            adOpenForwardOnly, adLockReadOnly, adCmdTableDirect
        
        ' get the values
        gstrInternetURL = gConfigRecord.Fields("InternetURL").Value
        gstrPublisherNetwork = gConfigRecord.Fields("PublisherNetwork").Value
        gstrPublisher = gConfigRecord.Fields("Publisher").Value
        gPublisherSecurityMode = _
            gConfigRecord.Fields("PublisherSecurityMode").Value
        gstrPublisherLogin = gConfigRecord.Fields("PublisherLogin").Value
        gstrPublisherPassword = gConfigRecord.Fields("PublisherPassword").Value
        gstrSubscriber = gConfigRecord.Fields("Subscriber").Value
        gstrHostName = gConfigRecord.Fields("HostName").Value
        gstrPublication = gConfigRecord.Fields("Publication").Value
        gstrIPAddress = gConfigRecord.Fields("IPAddress").Value
        gstrPublisherDatabase = gConfigRecord.Fields("PublisherDB").Value
        
        ' close it
        gConfigRecord.Close
    End If
    
    ' set the return value to True if there
    ' was no error
    LoadConfigs = Err.Number = 0

End Function


Public Function SSCEErrorDescription(nErr As Integer, sErrorDefine As String) As String

Dim sError As String
Dim sReason As String

' select the friendly error

Select Case nErr
    Case 25001
    sError = "SSCE_M_NOCURRENTRECORD"
    sReason = "Either not on a row or walked off the end of the row"
    
    Case 25002
    sError = "SSCE_M_INVALIDBUFFERSIZE"
    sReason = "Invalid buffer size specified"
    
    Case 25003
    sError = "SSCE_M_COLUMNLENGTHTOOBIG"
    sReason = "The specified length is too large for the column type"
    
    Case 25004
    sError = "SSCE_M_COLUMNNOTUPDATEABLE"
    sReason = "The column is not allowed to be modified"
    
    Case 25005
    sError = "SSCE_M_NULLINVALID"
    sReason = "NULL not allowed for this column"
    
    Case 25006
    sError = "SSCE_M_INVALIDOPERATION"
    sReason = "Attempt to perform operation is not valid for this object"
    
    Case 25007
    sError = "SSCE_M_INTRANSACTION"
    sReason = "Operation not valid when in a transaction"
    
    Case 25008
    sError = "SSCE_M_INVALIDPARAMETER"
    sReason = "An invalid parameter was passed to the storage engine"
    
    Case 25009
    sError = "SSCE_M_INVALIDPATH"
    sReason = "An invalid path name was specified"
    
    Case 25010
    sError = "SSCE_M_INVALIDFILENAME"
    sReason = "An invalid file name was specified"
    
    Case 25011
    sError = "SSCE_M_INVALIDDATABASE"
    sReason = "File examined was not recognized as a valid database format"
    
    Case 25012
    sError = "SSCE_M_UPDATENOTPREPARED"
    sReason = "An update was attempted before the PrepUpdate was called"
    
    Case 25013
    sError = "SSCE_M_TABLENOTLOCKED"
    sReason = "Operation is only supported on tables which are locked exclusively"
    
    Case 25014
    sError = "SSCE_M_SYSCOLACCESSDENIED"
    sReason = "Access to system column denied."
    
    Case 25015
    sError = "SSCE_M_CANTALLOCATEPAGE"
    sReason = "Could not allocate a page"
    
    Case 25016
    sError = "SSCE_M_KEYDUPLICATE"
    sReason = "Illegal attempt with a duplicate key"
    
    Case 25017
    sError = "SSCE_M_DATABASECORRUPTED"
    sReason = "Non database file or database file corrupted"
    
    Case 25018
    sError = "SSCE_M_WRITECONFLICT"
    sReason = "Write lock failed due to outstanding write lock"
    
    Case 25019
    sError = "SSCE_M_READCONFLICT"
    sReason = "Commit lock failed due to outstanding read lock"
    
    Case 25020
    sError = "SSCE_M_COMMITCONFLICT"
    sReason = "Read lock failed due to outstanding commit lock"
    
    Case 25021
    sError = "SSCE_M_SESSIONWRITECONFLICT"
    sReason = "Another session has a private version of the page"
    
    Case 25022
    sError = "SSCE_M_NULLKEYDISALLOWED"
    sReason = "Null keys are disallowed on index"
    
    Case 25023
    sError = "SSCE_M_UNKNOWNVERSION"
    sReason = "Unknown object version"
    
    Case 25024
    sError = "SSCE_M_CONNECTIONCONTROL"
    sReason = "Passive connection control prevented database open"
    
    Case 25025
    sError = "SSCE_M_INTEGRITYVIOLATION_MSTR"
    sReason = "References to key exist"
    
    Case 25026
    sError = "SSCE_M_INTEGRITYVIOLATION_SLV"
    sReason = "No referenced key exists"
    
    Case 25027
    sError = "SSCE_M_RECORDTOOBIG"
    sReason = "Record too big"
    
    Case 25028
    sError = "SSCE_M_INVALIDPASSWORD"
    sReason = "Invalid Password"
    
    Case 25029
    sError = "SSCE_M_RI_UPDATETWICE"
    sReason = "Attempt to update field in a record twice"
    
    Case 25030
    sError = "SSCE_M_RI_DUPLICATEKEY"
    sReason = "Duplicate key as a result of referential integrity action"
    
    Case 25031
    sError = "SSCE_M_RI_KEYNULLDISALLOWED"
    sReason = "Null value as a result of referential integrity action disallowed"
    
    Case 25032
    sError = "SSCE_M_RI_SESSIONWRITECONFLICT"
    sReason = "Session write conflict due to cascading"
    
    Case 25033
    sError = "SSCE_M_ILLEGALCOLLATINGSEQ"
    sReason = "Invalid sort sequence"
    
    Case 25034
    sError = "SSCE_M_FILELOCKVIOLATION"
    sReason = "File locking violation"
    
    Case 25035
    sError = "SSCE_M_FILESHAREVIOLATION"
    sReason = "File sharing violation"
    
    Case 25036
    sError = "SSCE_M_FILELOCKINGUNAVAILABLE"
    sReason = "File locking unavailable"
    
    Case 25037
    sError = "SSCE_M_DISKFULL"
    sReason = "No space left on disk"
    
    Case 25038
    sError = "SSCE_M_TEMPDISKFULL"
    sReason = "No space left on disk"
    
    Case 25039
    sError = "SSCE_M_FILEACCESSDENIED"
    sReason = "Access to the database file is not allowed"
    
    Case 25040
    sError = "SSCE_M_INVALIDAPPSIGNATURE"
    sReason = "Invalid application signature"
    
    Case 25042
    sError = "SSCE_M_INVALIDSORTDLL"
    sReason = "Invalid sort dll"
    
    Case 25043
    sError = "SSCE_M_TOOMANYACTIVEUSERS"
    sReason = "Too many active users"
    
    Case 25044
    sError = "SSCE_M_TOOMANYOPENFILES"
    sReason = "Too many open files"
    
    Case 25045
    sError = "SSCE_M_OBSOLETEDATABASEFORMAT"
    sReason = "Obsolete database format"
    
    Case 25046
    sError = "SSCE_M_FILENOTFOUND"
    sReason = "File not found"
    
    Case 25047
    sError = "SSCE_M_READVERIFYERROR"
    sReason = "Read verification error"
    
    Case 25048
    sError = "SSCE_M_NOMOREFILEHANDLES"
    sReason = "No more file handles"
    
    Case 25049
    sError = "SSCE_M_DISKNOTREADY"
    sReason = "Disk not ready"
    
    Case 25050
    sError = "SSCE_M_SHARINGBUFFEREXCEEDED"
    sReason = "Sharing buffer exceeded"
    
    Case 25051
    sError = "SSCE_M_DISKIO"
    sReason = "Disk IO error"
    
    Case 25052
    sError = "SSCE_M_THREADSNOTSUPPORTED"
    sReason = "This OS doesn't support threads"
    
    Case 25053
    sError = "SSCE_M_KEYNOTMADE"
    sReason = "No call to MakeKey"
    
    Case 25054
    sError = "SSCE_M_TOOMANYKEYS"
    sReason = "Too many columns in an index"
    
    Case 25055
    sError = "SSCE_M_CONTAINERNOTEMPTY"
    sReason = "Illegal attempt to delete a container that still had items"
    
    Case 25056
    sError = "SSCE_M_COLUMNWIDTHNOTEVEN"
    sReason = "The size of the UNICODE text column is not even"
    
    Case 25057
    sError = "SSCE_M_LV_INVALIDAPPEND"
    sReason = "Cannot append long value"
    
    Case 25058
    sError = "SSCE_M_NOCURRENTINDEX"
    sReason = "Invalid operation with out a current index"
    
    Case 25059
    sError = "SSCE_M_ILLEGALINDEXCOLUMN"
    sReason = "Column's type not indexable"
    
    Case 25060
    sError = "SSCE_M_REFERENCENOTFOUND"
    sReason = "No such reference"
    
    Case 25061
    sError = "SSCE_M_INTERNAL_INVALIDDBID"
    sReason = "Internal error"
    
    Case 25062
    sError = "SSCE_M_OBJECTNOTREPLICABLE"
    sReason = "The object is not replicable"
    
    Case 25064
    sError = "SSCE_M_ONLYONESPECIALCOLUMN"
    sReason = "Illegal attempt to create a second Row GUID,Autoinc,or Version on column"
    
    Case 25065
    sError = "SSCE_M_TOOMANYINDEXES"
    sReason = "Too many indexes"
    
    Case 25066
    sError = "SSCE_M_TOOMANYCOLUMNS"
    sReason = "Too many columns"
    
    Case 25067
    sError = "SSCE_M_TRANSTOODEEP"
    sReason = "Transaction nested too deeply"
    
    Case 25068
    sError = "SSCE_M_SELFREFERENCE"
    sReason = "Referencing or referenced index is the same"
    
    Case 25069
    sError = "SSCE_M_NOPRIMARYINDEX"
    sReason = "No primary index on referenced table"
    
    Case 25070
    sError = "SSCE_M_DIFFNUMRELCOLUMNS"
    sReason = "Different number of columns in relationship"
    
    Case 25071
    sError = "SSCE_M_CONFLICTINGREFERENCE"
    sReason = "Reference conflicts with existing reference"
    
    Case 25072
    sError = "SSCE_M_PRIMARYKEYNULLCONFLICT"
    sReason = "Primary keys must be created on NON-NULL columns"
    
    Case 25073
    sError = "SSCE_M_TABLEMUSTHAVECOLUMNS"
    sReason = "Tables must have at least one column"
    
    Case 25074
    sError = "SSCE_M_DEFAULTEXPRTOOLONG"
    sReason = "The expression string for the default value is too long"
    
    Case 25075
    sError = "SSCE_M_DISALLOWIDENTITYNULL"
    sReason = "Can not create an identity on column which allows nulls"
    
    Case 25077
    sError = "SSCE_M_CONSTRAINTINUSE"
    sReason = "Can not remove constraint,it is currently referenced by another constraint"
    
    Case 25078
    sError = "SSCE_M_INDEXINUSE"
    sReason = "Can not remove index,it is currently used to enforce a constraint"
    
    Case 25079
    sError = "SSCE_M_ENCRYPTEDDBMUSTHAVEPWD"
    sReason = "Encrypted Databases MUST have a password"
    
    Case 25080
    sError = "SSCE_M_NOOSENCRYPTIONSUPPORT"
    sReason = "The CryptoAPI isn't available on this operating system"
    
    Case 25081
    sError = "SSCE_M_VIOLATEDAUTOINCRANGE"
    sReason = "AutoInc generated was outside the min/max range specified for the column"
    
    Case 25082
    sError = "SSCE_M_UNIQUENULLCONFLICT"
    sReason = "Unique indexes/constraints cannot be created over nullable columns."
    
    Case 25083
    sError = "SSCE_M_CYCLEDETECTED"
    sReason = "Reference would cause a cycle - not allowed"
    
    Case 25084
    sError = "SSCE_M_RELCOLUMNSTYPEMISMATCH"
    sReason = "Columns in relationship do not match due to type"
    
    Case 25085
    sError = "SSCE_M_PKDUPLICATE"
    sReason = "The table already has a primary key"
    
    Case 25086
    sError = "SSCE_M_INVALIDINCREMENT"
    sReason = "Identity column contains invalid INCREMENT."
    
    Case 25200
    sError = "SSCE_WRN_COLUMNNULL"
    sReason = "Column is NULL-valued"
    
    Case 25201
    sError = "SSCE_WRN_BUFFERTRUNCATED"
    sReason = "Buffer too short,data truncated"
    
    Case 25202
    sError = "SSCE_WRN_SEEKNOTEQUAL"
    sReason = "SeekLE or SeekGE did not find exact match"
    
    Case 25203
    sError = "SSCE_WRN_OPTIONSIGNORED"
    sReason = "Some options specified where ignored"
    
    Case 25204
    sError = "SSCE_WRN_COLUMNDATATRUNCATED"
    sReason = "Data truncated"
    
    Case 25205
    sError = "SSCE_WRN_DATABASEENCRYPTED"
    sReason = "Database is encrypted"
    
    Case 25206
    sError = "SSCE_WRN_RI_RECORDSUPDATED"
    sReason = "Some additional records got updated due to referential integrity actions"
    
    Case 25207
    sError = "SSCE_WRN_RI_RECORDSDELETED"
    sReason = "Some additional records got deleted due to referential integrity actions"
    
    Case 25208
    sError = "SSCE_WRN_ROWLOCKCONNECTFAILED"
    sReason = "Requested row level locking,but unable to honor request"
    
    Case 25209
    sError = "SSCE_WRN_PAGELOCKCONNECTFAILED"
    sReason = "Requested page level locking,but unable to honor request"
    
    Case 25500
    sError = "SSCE_M_QP_BADOPT"
    sReason = "Query optimization error,plan not created or found"
    
    Case 25501
    sError = "SSCE_M_QP_BADPARSE"
    sReason = "Error parsing query"
    
    Case 25502
    sError = "SSCE_M_QP_SETLISTCOUNT"
    sReason = "Count of column names does not match up with count of source expressions"
    
    Case 25503
    sError = "SSCE_M_QP_BAD_COLNAME"
    sReason = "Bad column name specified"
    
    Case 25504
    sError = "SSCE_M_QP_INSERTMISSINGCOL"
    sReason = "Integrity violation,not an identity column or does not have the nullability"
    
    Case 25505
    sError = "SSCE_M_QP_BADCNSTS"
    sReason = "Bad constraint specified"
    
    Case 25506
    sError = "SSCE_M_QP_NOMEMORY"
    sReason = "ERROR : Out of memory!"
    
    Case 25507
    sError = "SSCE_M_QP_UNCLOSEDCOMMENT"
    sReason = "Process premature EOF"
    
    Case 25508
    sError = "SSCE_M_QP_TOKTOOLONG"
    sReason = "Token specified is too long"
    
    Case 25509
    sError = "SSCE_M_QP_ATOF_OVERFLOW"
    
    sReason = "Function overflow,ATOF"
    Case 25510
    
    sError = "SSCE_M_QP_NUME_OUTOFRANGE"
    sReason = "Numeric value overflow during parsing"
    
    Case 25511
    sError = "SSCE_M_QP_UNCLOSEDQUOTE"
    sReason = "Found end-of-query before end-quote"
    
    Case 25512
    sError = "SSCE_M_QP_BADMONEY"
    
    sReason = "Bad money value specified"
    Case 25513
    
    sError = "SSCE_M_QP_IDENTWITHNULL"
    sReason = "Invalid identifier,NULLs specified"
    
    Case 25514
    sError = "SSCE_M_QP_NAME_EMPTY"
    sReason = "Invalid identifier,empty string specified"
    
    Case 25515
    sError = "SSCE_M_QP_NO_SCALAR_IN_SELECT"
    sReason = "For aggregates or grouping,the SELECT clause can only contain aggregates or"
    
    Case 25516
    sError = "SSCE_M_QP_NO_AGG_IN_AGG"
    sReason = "Aggregates cannot be nested within other aggregates"
    
    Case 25517
    sError = "SSCE_M_QP_NO_SCALAR_IN_HAVING"
    sReason = "For aggregates or grouping,the HAVING clause can only contain aggregates or"
    
    Case 25518
    sError = "SSCE_M_QP_NO_SCALAR_IN_AGGRORDERBY"
    sReason = "For aggregates or grouping,the ORDER BY clause can only contain aggregates or"
    
    Case 25519
    sError = "SSCE_M_QP_NO_AGG_IN_GROUPBY"
    sReason = "No aggregates in expressions in the GROUP BY list"
    
    Case 25520
    sError = "SSCE_M_QP_NO_AGG_IN_ORDERBY"
    sReason = "No aggregates in expressions in the ORDER BY list"
    
    Case 25521
    sError = "SSCE_M_QP_NO_DUPEXPR_IN_ORDERBY"
    sReason = "Duplicate expression specified in ORDER BY list"
    
    Case 25522
    sError = "SSCE_M_QP_DUP_TABLE_ALIAS"
    sReason = "Not all table aliases in the query are distinct"
    
    Case 25523
    sError = "SSCE_M_QP_DUP_COLUMN_ALIAS"
    sReason = "Not all column aliases are unique"
    
    Case 25524
    sError = "SSCE_M_QP_HAVING_WITHOUT_GROUPBY"
    sReason = "Cannot have a HAVING clause without a GROUP BY (implicit or explicit)"
    
    Case 25525
    sError = "SSCE_M_QP_NO_AGG_IN_WHERE"
    sReason = "The WHERE clause cannot reference aggregates"
    
    Case 25526
    sError = "SSCE_M_QP_DUP_CNST"
    sReason = "Duplicate constraint specified"
    
    Case 25527
    sError = "SSCE_M_QP_AMBIGUOUS_FUNC"
    sReason = "A method in the array that matches this method already exists,so this is an"
    
    Case 25529
    sError = "SSCE_M_QP_CANT_COCREATE"
    sReason = "Can not instantiate the OLE DB provider"
    
    Case 25530
    sError = "SSCE_M_QP_SETLIST_DUPCOLNAME"
    sReason = "Column name has already been specified"
    
    Case 25531
    sError = "SSCE_M_QP_AMBIGUOUS_COLNAME"
    sReason = "Ambiguity in resolution of column name"
    
    Case 25532
    sError = "SSCE_M_QP_BAD_FLOAT_PRECISION"
    sReason = "Bad float precision specified"
    
    Case 25533
    sError = "SSCE_M_QP_LONGIDNAME"
    sReason = "Identifier specified is too long"
    
    Case 25534
    sError = "SSCE_M_QP_LOB_IN_ORDERBY"
    sReason = "Order by operators do not allow large objects (i.e. ntext/image)"
    
    Case 25536
    sError = "SSCE_M_QP_INVALIDMETA_FOR_LOB"
    sReason = "Invalid meta data for large object"
    
    Case 25537
    sError = "SSCE_M_QP_SUBQUERY_COLUMN_MISMATCH"
    sReason = "Sub-query column mismatch"
    
    Case 25538
    sError = "SSCE_M_QP_ENCRYPTION_NO_DBPASSWORD"
    sReason = "Encryption specified,but no database password specified"
    
    Case 25539
    sError = "SSCE_M_QP_BADLIKEESCCHAR"
    sReason = "Invalid escape character for LIKE predicate"
    
    Case 25540
    sError = "SSCE_M_QP_BADDATEOPTION"
    sReason = "Bad date option specified"
    
    Case 25541
    sError = "SSCE_M_QP_INVALID_IDENTIFIER"
    sReason = "Invalid identifier specified"
    
    Case 25542
    sError = "SSCE_M_QP_READONLYINSERT"
    sReason = "Attempt to insert into a read only table"
    
    Case 25543
    sError = "SSCE_M_QP_READONLYUPDATE"
    sReason = "Attempt to update a read only table"
    
    Case 25544
    sError = "SSCE_M_QP_READONLYDELETE"
    sReason = "Attempt to delete from a read only table"
    
    Case 25545
    sError = "SSCE_M_QP_NOINSERTCOLUMNS"
    sReason = "No values provided for INSERT statement into table <table_name>"
    
    Case 25546
    sError = "SSCE_M_QP_MODIFYPROTECTEDCOL"
    sReason = "Attempt to modify a non-writeable column <column_name>"
    
    Case 25547
    sError = "SSCE_M_QP_SCALARORDERBYNOTSUPPORTED"
    sReason = "ORDER BY <column number> not supported"
    
    Case 25548
    sError = "SSCE_M_QP_LOB_IN_DISTINCT"
    sReason = "Distinct operator does not work with large objects (i.e. ntext/image)"
    
    Case 25549
    sError = "SSCE_M_QP_LOB_IN_GROUPBY"
    sReason = "Group by operator does not work with large objects (i.e. ntext/image)"
    
    Case 25550
    sError = "SSCE_M_QP_MULTI_IDENTITY_COLUMNS"
    sReason = "Multiple identity columns specified for table. Only one identity column per"
    
    Case 25551
    sError = "SSCE_M_QP_BAD_IDENTITY_TYPE"
    sReason = "Identity column must be of data type int,and constrained to be nonnullable."
    
    Case 25552
    sError = "SSCE_M_QP_GROUPBY_MUST_REFERENCE_COLS"
    sReason = "Elements in the GROUP BY clause must reference columns in the SELECT list"
    
    Case 25553
    sError = "SSCE_M_QP_CASCADEONSELFREFFK"
    sReason = "Can't have foreign key cascade RI on self-table keys"
    
    Case 25554
    sError = "SSCE_M_QP_NODEFAULTONIDENTITYCOL"
    sReason = "Defaults cannot be created on columns with an IDENTITY attribute."
    
    Case 25555
    sError = "SSCE_M_QP_SUBQUERY_NOLOBCOLUMNS"
    sReason = "Sub-query columns cannot be BLOB/CLOB"
    
    Case 25556
    sError = "SSCE_M_QP_INVALID_CHARACTER_IN_IDENTIFIER"
    sReason = "Names cannot begin with a number sign (#)."
    
    Case 25900
    sError = "SSCE_M_QP_BADEXPR"
    sReason = "Internal error in expression service"
    
    Case 25901
    sError = "SSCE_M_QP_OVFLO"
    sReason = "Expression overflow"
    
    Case 25902
    sError = "SSCE_M_QP_EXPRTYPECHECK"
    sReason = "Type mismatch in IN expression"
    
    Case 25903
    sError = "SSCE_M_QP_DATEPART"
    sReason = "Error in part of date format"
    
    Case 25904
    sError = "SSCE_M_QP_DATEABORT"
    sReason = "Error in date format"
    
    Case 25905
    sError = "SSCE_M_QP_SYNTAX"
    sReason = "A syntax error occurred"
    
    Case 25906
    sError = "SSCE_M_QP_CONVSYNTAX"
    sReason = "Syntax error in conversion logic"
    
    Case 25907
    sError = "SSCE_M_QP_MONEYOVERFLOW"
    sReason = "Overflow of money data type in string to money conversion"
    
    Case 25909
    sError = "SSCE_M_QP_NOCOLUMNREFINDEFAULT"
    sReason = "Column references in default values not supported"
    
    Case 25910
    sError = "SSCE_M_QP_NOAGGRINDEFAULT"
    sReason = "Aggregate functions in default values not supported"
    
    Case 25911
    sError = "SSCE_M_QP_NOAGGRONLOB"
    sReason = "Can not put large objects (blob/ntext) in aggregates"
    
    Case 25912
    sError = "SSCE_M_QP_NOCONVDUETOTRUNC"
    sReason = "Can not convert due to truncation of original data"
    
    Case 25913
    sError = "SSCE_M_QP_ARITHOPNOTSUPPORTED"
    sReason = "Arithmetic operator is not supported,first parameter indicies operator"
    
    Case 25914
    sError = "SSCE_M_QP_NOMODULOSUPPORT"
    sReason = "Modulo is not supported on real,float,money,and numeric types"
    
    Case 25916
    sError = "SSCE_M_QP_SUBQUERYINVALIDOP"
    sReason = "Invalid operator against result type for subquery expression,(>,<,etc.)"
    
    Case 25917
    sError = "SSCE_M_QP_NOBITOPSUPPORT"
    sReason = "Bit operators (&,|,^,~) are not supported on real,float,money and numeric"
    
    Case 25918
    sError = "SSCE_M_QP_BADARITHTYPE"
    sReason = "Trying to do arithmetic operation with invalid type"
    
    Case 25919
    sError = "SSCE_M_QP_BADARITHOP"
    sReason = "Arithmetic operator is not recognized by SSCE"
    
    Case 25920
    sError = "SSCE_M_QP_TRUNCATION"
    sReason = "Data truncation error"
    
    Case 25921
    sError = "SSCE_M_QP_INVALIDFUNCTIONNAME"
    sReason = "Not a recognized function name"
    
    Case 25922
    sError = "SSCE_M_QP_INVALIDFUNCTIONARGS"
    sReason = "Bad arguments to function <func_name>"
    
    Case 25923
    sError = "SSCE_M_QP_NOSUPPORTFORLOB"
    sReason = "Data types ntext and image cannot be used in WHERE,HAVING,GROUP BY,ON or IN"
    
    Case 25924
    sError = "SSCE_M_QP_BINDTSYNTAX"
    sReason = "Could not convert binary to datetime due to possible overflow"
    
    Case 25925
    sError = "SSCE_M_QP_NUMDTSYNTAX"
    sReason = "Numeric value overflowed datetime value during conversion"
    
    Case 25926
    sError = "SSCE_M_QP_MONDTSYNTAX"
    sReason = "Money value overflowed datetime value during conversion"
    
    Case 25927
    sError = "SSCE_M_QP_BIGINTDTSYNTAX"
    sReason = "Bigint value overflowed datetime value during conversion"
    
    Case 25928
    sError = "SSCE_M_QP_TOOMANYARGS"
    sReason = "Too many arguments supplied to the function"
    
    Case 25929
    sError = "SSCE_M_QP_UMINUSNUMONLY"
    sReason = "Can not perform a unary minus on types other than"
    
    Case 25930
    sError = "SSCE_M_QP_PRECISIONMISMATCH"
    sReason = "Can't perform operation due to numeric precision mismatch"
    
    Case 25931
    sError = "SSCE_M_QP_SCALEMISMATCH"
    sReason = "Can't perform operation due to scale mismatch"
    
    Case 25932
    sError = "SSCE_M_QP_NOIMPLICITCONVALLOWED"
    sReason = "Can not perform an implicit conversion from one type to another"
    
    Case 25933
    sError = "SSCE_M_QP_DATETIMEOVFLO"
    sReason = "Conversion of string to datetime caused an overflow"
    
    Case 25934
    sError = "SSCE_M_QP_MISSINGTYPELOGIC"
    sReason = "Error thrown when hitting a code path that requires type specific logic that"
    
    Case 25935
    sError = "SSCE_M_QP_DIVBYZERO"
    sReason = "Divide by zero attempt"
    
    Case 25937
    sError = "SSCE_M_QP_NOTIMEPARTSPECIFIED"
    sReason = "Syntax error converting datetime from character string. No TIME part was"
    
    Case 25938
    sError = "SSCE_M_QP_INVALIDHOURTIMEPARTSPECIFIED"
    sReason = "Syntax error converting datetime from character string. Invalid HOUR specified"
    
    Case 25939
    sError = "SSCE_M_QP_INVALIDMINUTETIMEPARTSPECIFIED"
    sReason = "Syntax error converting datetime from character string. Invalid MINUTE"
    
    Case 25940
    sError = "SSCE_M_QP_INVALIDSECONDIMEPARTSPECIFIED"
    sReason = "Syntax error converting datetime from character string. Invalid SECOND"
    Case 25941
    
    sError = "SSCE_M_QP_INVALIDMSEDCONDIMEPARTSPECIFIED"
    sReason = "Syntax error converting datetime from character string. Invalid MILISECOND"
    
    Case 25942
    sError = "SSCE_M_QP_SYNTAXERRORINDATETIMESTRING"
    sReason = "Syntax error converting datetime from character string. Invalid syntax in DATE"
    
    Case 25943
    sError = "SSCE_M_QP_FORMATERRORINDATETIMESTRING"
    sReason = "Format error converting datetime from character string. Invalid format of DATE"
    
    Case 25944
    sError = "SSCE_M_QP_INVALIDORDERINDATETIMESTRING"
    sReason = "Format error converting datetime from character string. Invalid order within"
    
    Case 26100
    sError = "SSCE_M_QP_BADTABLE"
    sReason = "Invalid table name"
    
    Case 26101
    sError = "SSCE_M_QP_BADSCHEMAROWSET"
    sReason = "Error accessing schema rowset"
    
    Case 26102
    sError = "SSCE_M_QP_TBLSCHEMAROWSET"
    sReason = "Error accessing tables schema rowset"
    
    Case 26300
    sError = "SSCE_M_QP_OLEDBERR"
    sReason = "Error returned from OLE-DB call"
    
    Case 26301
    sError = "SSCE_M_QP_BADADT"
    sReason = "Cannot cast from the specified type"
    
    Case 26302
    sError = "SSCE_M_QP_BADTYPE"
    sReason = "Bad type specified"
    
    Case 26303
    sError = "SSCE_M_QP_INTERNAL"
    sReason = "An internal query processor error occurred"
    
    Case 26304
    sError = "SSCE_M_QP_ALLWELL"
    sReason = "No error"
    
    Case 26305
    sError = "SSCE_M_QP_BADADTMETAINFO"
    sReason = "Bad meta information"
    
    Case 26306
    sError = "SSCE_M_QP_COERSIONERROR"
    sReason = "Failed data conversion"
    
    Case 26307
    sError = "SSCE_M_QP_BADQUERY"
    sReason = "UNDONE - remove this error"
    
    Case 26308
    sError = "SSCE_M_QP_ZEROLENGTHBINARYCONST"
    sReason = "SSCE does not support zero length binary constants"
    
    Case 27000
    sError = "SSCE_M_TOOMANYSESSIONSOUTSTANDING"
    sReason = "Too many outstanding sessions"
    
    Case 27001
    sError = "SSCE_M_PRIMARYKEYPROPERTYNOTSETTABLE"
    sReason = "Can not set COLUMN property PRIMARY KEY when adding a column. Create a table"
    
    Case 27002
    sError = "SSCE_M_UNIQUEKEYPROPERTYNOTSETTABLE"
    sReason = "Can not set COLUMN property UNIQUE when adding a column. Create a table level"
    
    Case 27003
    sError = "SSCE_M_COLUMNNULLABILITYNOTALTERABLE"
    sReason = "Can not change a column's nullability after it has been created."
    
    Case 27004
    sError = "SSCE_M_CANTMODIFYIDENTITYATTRIBUTE"
    sReason = "Can not add the IDENTITY attribute to column after it has been created."
    
    Case 27005
    sError = "SSCE_M_INVALIDCOLUMNPROPERTY"
    sReason = "Attempt to set invalid column property."
    
    Case 27500
    sError = "SSCE_M_DECIMALOVERFLOW"
    sReason = "Decimal data overflow error"
    
    Case 27501
    sError = "SSCE_M_DECIMALTRUNCATED"
    sReason = "Decimal data truncation error"
    
    Case 27502
    sError = "SSCE_M_CMN_INVALIDPARAMETER"
    sReason = "An invalid parameter was passed to Common"
    
    Case 27700
    sError = "SSCE_WRN_DECIMALTRUNCATED"
    sReason = "Decimal data truncation warning"
    
    Case 28000
    sError = "SSCE_M_OPENTWICE"
    sReason = "SSCE Transport Open called without closing previous open"
    
    Case 28001
    sError = "SSCE_M_INTERNETREADERROR"
    sReason = "InternetRead returned an incorrect number of bytes"
    
    Case 28002
    sError = "SSCE_M_INTERNETWRITEERROR"
    sReason = "InternetWrite wrote an incorrect number of bytes"
    
    Case 28003
    sError = "SSCE_M_INVALIDRSCBID"
    sReason = "Either the server is out of memory or an incorrect RSCBID was sent in a"
    
    Case 28004
    sError = "SSCE_M_TOOMANYPENDINGREQUESTS"
    sReason = "There are too many requests pending at the server,no further requests can be"
    
    Case 28005
    sError = "SSCE_M_COULDNTSTARTRECTHREAD"
    sReason = "Failed trying to start a thread to host the reconciler"
    
    Case 28006
    sError = "SSCE_M_CANTLOADRECRESOURCEDLL"
    sReason = "Failed trying to load the reconciler resource dll"
    
    Case 28007
    sError = "SSCE_M_INVALIDRECONCILERPARAM"
    sReason = "Invalid parameter to Reconciler"
    
    Case 28008
    sError = "SSCE_M_INVALIDTRNSPTCOMMAND"
    sReason = "Invalid transport command"
    
    Case 28009
    sError = "SSCE_M_CANTCOCREATERECONCILER"
    sReason = "Unable to cocreate an instance of the Reconciler"
    
    Case 28010
    sError = "SSCE_M_CANTCOCREATERECONERROR"
    sReason = "Unable to cocreate an instance of the Reconcilers errors object"
    
    Case 28011
    sError = "SSCE_M_HTTPSTATUSDENIED"
    sReason = "Authentication failed on server"
    
    Case 28012
    sError = "SSCE_M_HTTPSTATUSPROXYAUTHREQ"
    sReason = "Authentication failed on proxy server"
    
    Case 28013
    sError = "SSCE_M_HTTPSTATUSBADREQUEST"
    sReason = "Bad request sent to server"
    
    Case 28014
    sError = "SSCE_M_HTTPSTATUSPAYMENTREQ"
    sReason = "Payment required to access this site"
    
    Case 28015
    sError = "SSCE_M_HTTPSTATUSFORBIDDEN"
    sReason = "Forbidden from accessing this site without SSL (https)"
    
    Case 28016
    sError = "SSCE_M_HTTPSTATUSNOTFOUND"
    sReason = "URL not found"
    
    Case 28017
    sError = "SSCE_M_HTTPSTATUSBADMETHOD"
    sReason = "Bad method invoked"
    
    Case 28018
    sError = "SSCE_M_HTTPSTATUSNONEACCEPT"
    sReason = "Server returned 'none acceptable'"
    
    Case 28019
    sError = "SSCE_M_HTTPSTATUSREQTIMEOUT"
    sReason = "Request timed out on server"
    
    Case 28020
    sError = "SSCE_M_HTTPSTATUSCONFLICT"
    sReason = "Server returned status 'Conflict'"
    
    Case 28021
    sError = "SSCE_M_HTTPSTATUSGONE"
    sReason = "Server returned status 'Gone'"
    
    Case 28022
    sError = "SSCE_M_HTTPSTATUSSERVERERROR"
    sReason = "Server Error"
    
    Case 28023
    sError = "SSCE_M_HTTPSTATUSNOTSUPPORTED"
    sReason = "Server returned status 'not supported'"
    
    Case 28024
    sError = "SSCE_M_HTTPSTATUSBADGATEWAY"
    sReason = "Bad Gateway"
    
    Case 28025
    sError = "SSCE_M_HTTPSTATUSSVCUNAVAIL"
    sReason = "Service unavailable"
    
    Case 28026
    sError = "SSCE_M_HTTPSTATUSGATTIMEOUT"
    sReason = "Gateway timeout"
    
    Case 28027
    sError = "SSCE_M_HTTPSNOTSUPPORTED"
    sReason = "HTTPS not supported on this OS (Birch)"
    
    Case 28028
    sError = "SSCE_M_REPLPROHIBITED"
    sReason = "Replication is prohibited for this server agent"
    
    Case 28029
    sError = "SSCE_M_RDAPROHIBITED"
    sReason = "RDA is prohibited for this server agent"
    
    Case 28030
    sError = "SSCE_M_COMPRESSIONFAILED"
    sReason = "Compression failed"
    
    Case 28031
    sError = "SSCE_M_DECOMPRESSIONFAILED"
    sReason = "DeCompression failed"
    
    Case 28032
    sError = "SSCE_M_FAILUREINITIALIZINGINTERNET"
    sReason = "Failure initializing internet functionality"
    
    Case 28033
    sError = "SSCE_M_INVALIDURLSYNTAX"
    sReason = "Invalid URL syntax"
    
    Case 28034
    sError = "SSCE_M_INTERNETCONNECTFAILURE"
    sReason = "Internet Connect Failure"
    
    Case 28035
    sError = "SSCE_M_FAILUREQUERYINGHEADER"
    sReason = "Failure when querying header"
    
    Case 28036
    sError = "SSCE_M_CLOSEINTERNETHANDLEFAILED"
    sReason = "Failure when closing an internet handle"
    
    Case 28037
    sError = "SSCE_M_HTTPSENDREQUESTFAILED"
    sReason = "HttpSendRequest failed,hresult has more detail"
    
    Case 28500
    sError = "SSCE_M_CANTCOCREATESSCEERRORS"
    sReason = "Unable to cocreate an instance of the SSCE Errors collection object"
    
    Case 28501
    sError = "SSCE_M_SUBSCRIBERDATABASEPATHMISSING"
    sReason = "Subscriber Database Path is missing"
    
    Case 28502
    sError = "SSCE_M_INTERNETURLPROPERTYMISSING"
    sReason = "Internet URL property is missing"
    
    Case 28503
    sError = "SSCE_M_PUBLISHERPROPERTYMISSING"
    sReason = "Publisher property is missing"
    
    Case 28504
    sError = "SSCE_M_PUBLISHERDATABASEPROPERTYMISSING"
    sReason = "Publisher database property is missing"
    
    Case 28505
    sError = "SSCE_M_PUBLICATIONPROPERTYMISSING"
    sReason = "Publication property is missing"
    
    Case 28506
    sError = "SSCE_M_SUBSCRIBERPROPERTYMISSING"
    sReason = "Subscriber property is missing"
    
    Case 28507
    sError = "SSCE_M_MISSINGDISTRIBUTORPROPERTIES"
    sReason = "Some distributor properties are missing"
    
    Case 28508
    sError = "SSCE_M_MISSINGDISTRIBUTORADDRESS"
    sReason = "Missing distributor address"
    
    Case 28509
    sError = "SSCE_M_MISSINGPUBLISHERADDRESS"
    sReason = "Missing publisher address"
    
    Case 28510
    sError = "SSCE_M_INVALIDPUBLISHERSECURITYMODE"
    sReason = "Invalid publisher security mode"
    
    Case 28511
    sError = "SSCE_M_INVALIDDISTRIBUTORSECURITYMODE"
    sReason = "Invalid distributor security mode"
    
    Case 28512
    sError = "SSCE_M_INVALIDQUERYTIMEOUTVALUE"
    sReason = "Invalid query timeout value"
    
    Case 28513
    sError = "SSCE_M_INVALIDLOGINTIMEOUTVALUE"
    sReason = "Invalid login timeout value"
    
    Case 28514
    sError = "SSCE_M_INVALIDDISTRIBUTORNETWORK"
    sReason = "Invalid distributor network"
    
    Case 28515
    sError = "SSCE_M_INVALIDEXCHANGETYPE"
    sReason = "Invalid exchange type"
    
    Case 28516
    sError = "SSCE_M_INVALIDVALIDATIONTYPE"
    sReason = "Invalid validation type"
    
    Case 28517
    sError = "SSCE_M_INITIALIZEFAILED"
    sReason = "Initialize failed"
    
    Case 28518
    sError = "SSCE_M_ADDSUBSCRIPTIONFAILED"
    sReason = "Add Subscription failed"
    
    Case 28519
    sError = "SSCE_M_UNSUPPORTEDDBDROPOPTION"
    sReason = "Unsupported Database Drop option"
    
    Case 28520
    sError = "SSCE_M_DROPSUBSCRIPTIONFAILED"
    sReason = "Drop Subscription failed"
    
    Case 28521
    sError = "SSCE_M_DBALREADYREPLICABLE"
    sReason = "The Database is already replicable"
    
    Case 28522
    sError = "SSCE_M_RUNFAILED"
    sReason = "Run failed"
    
    Case 28523
    sError = "SSCE_M_INITIALIZATIONFAILED"
    sReason = "Internal object initialization failed"
    
    Case 28524
    sError = "SSCE_M_TERMINATEFAILED"
    sReason = "Terminate failed"
    
    Case 28525
    sError = "SSCE_M_REINITIALIZESUBSCRIPTIONFAILED"
    sReason = "ReInitialize subscription failed"
    
    Case 28526
    sError = "SSCE_M_UNEXPECTEDROCENCOUNTERED"
    sReason = "Unexpected ROC encountered in message"
    
    Case 28527
    sError = "SSCE_M_COLUMNORDINALNOTFOUND"
    sReason = "Column ordinal not found"
    
    Case 28528
    sError = "SSCE_M_NICKNAMEGENERATIONFAILED"
    sReason = "Failed to generate a nickname"
    
    Case 28529
    sError = "SSCE_M_DBNOTREPLICABLE"
    sReason = "Database is not replicable"
    
    Case 28530
    sError = "SSCE_M_FAILUREREADINGREGISTRY"
    sReason = "Failure trying to read registry"
    
    Case 28531
    sError = "SSCE_M_FAILUREWRITINGREGISTRY"
    sReason = "Failure trying to write to the registry"
    
    Case 28532
    sError = "SSCE_M_TRANSPORTREADRETURNEDNODATA"
    sReason = "Read from transport returned no data"
    
    Case 28533
    sError = "SSCE_M_CANTCOCREATESQLSERVERCE"
    sReason = "Can't cocreate instance of SQLSERVERCE"
    
    Case 28534
    sError = "SSCE_M_RDAPULLFAILED"
    sReason = "RDA Pull failed"
    
    Case 28535
    sError = "SSCE_M_RDAPUSHFAILED"
    sReason = "RDA Push failed"
    
    Case 28536
    sError = "SSCE_M_RDASUBMITSQLFAILED"
    sReason = "RDA SubmitSQL failed"
    
    Case 28537
    sError = "SSCE_M_IDENTITYRANGE"
    sReason = "Failure establishing Identity Range"
    
    Case 28538
    sError = "SSCE_M_INVALIDARTICLE"
    sReason = "An article in the message is invalid"
    
    Case 28539
    sError = "SSCE_M_OBJECTALREADYINITIALIZED"
    sReason = "The object is already initialized"
    
    Case 28540
    sError = "SSCE_M_OBJECTNOTINITIALIZED"
    sReason = "The object is not initialized"
    
    Case 28541
    sError = "SSCE_M_LOCALTABLENAMEMISSING"
    sReason = "Local table name missing on Push call"
    
    Case 28542
    sError = "SSCE_M_NOTALLPARAMSSETFORPULL"
    sReason = "Not all the parameters were set for the pull operation"
    
    Case 28543
    sError = "SSCE_M_TABLENOTTRACKED"
    sReason = "The Table is not an RDA tracked table"
    
    Case 28544
    sError = "SSCE_M_RDAERRORROWSRETURNED"
    sReason = "One or more error rows returned from RDA PUSH,see associated _err table"
    
    Case 28545
    sError = "SSCE_M_SUBSCRIBERCONNECTIONSTRINGMISSING"
    sReason = "You must supply a valid oledb connection string"
    
    Case 28546
    sError = "SSCE_M_INVALIDCONNECTIONSTRING"
    sReason = "The connection string cannot be parsed"
    
    Case 28547
    sError = "SSCE_M_DATABASEALREADYOPEN"
    sReason = "Cannot compact a database that is open"
    
    Case 28548
    sError = "SSCE_M_PARAMETERTOOLARGE"
    sReason = "A Parameter specified is too large"
    
    Case 28549
    sError = "SSCE_M_FAILEDRETRIES"
    sReason = "Unable to apply retries"
    
    Case 28550
    sError = "SSCE_M_COMPACTFAILED"
    sReason = "Unable to compact database"
    
    Case 28551
    sError = "SSCE_M_MISSINGCONNECTSTRING"
    sReason = "Missing connection string for either Pull or Push"
    
    Case 28552
    sError = "SSCE_M_ROW_RUID"
    sReason = "No ROWGUID column for this table"
    
    Case 28553
    sError = "SSCE_M_SPECIAL_COLUMN"
    sReason = "NO ROWGUID,or Identity range"
    
    Case 28554
    sError = "SSCE_M_VALIDATEFAILED"
    sReason = "The number of rows for the table do not match SQL-Server"
    
    Case 28555
    sError = "SSCE_M_FAILEDMAKEDBREP"
    sReason = "Failed making database replicable/unreplicable,reinitialize is now required"
    
    Case 28556
    sError = "SSCE_M_TABLENAMEMISSING"
    sReason = "A Table name is required for the Pull command"
    
    Case 28557
    sError = "SSCE_M_UNUSABLEDATABASE"
    sReason = "The database is in an unusable state. Delete it and recreate it."
    
    Case 28558
    sError = "SSCE_M_CREATEDATABASE"
    sReason = "Problems attempting to create the database"
    
    Case 28559
    sError = "SSCE_M_OPENDATABASE"
    sReason = "Problems attempting to open the database"
    
    Case 28560
    sError = "SSCE_M_EXECUTEFAILED"
    sReason = "OLEDB Execute Method failed,Bad or Invalid SQL statement"
    
    Case 28561
    sError = "SSCE_M_ROLLBACKFAILED"
    sReason = "Failed to ROLLBACK changes"
    
    Case 28562
    sError = "SSCE_M_PROCESSDELETEFAILED"
    sReason = "Failed to create the Delete message to send to the server"
    
    Case 28563
    sError = "SSCE_M_CLIENTAGENT_NOMEMORY"
    sReason = "Unsufficient memory on the device"
    
    Case 28564
    sError = "SSCE_M_DELETEFAILED"
    sReason = "Failed to DELETE a row"
    
    Case 28565
    sError = "SSCE_M_INSERTFAILED"
    sReason = "Failed to INSERT a row"
    
    Case 28566
    sError = "SSCE_M_UPDATEFAILED"
    sReason = "Failed to UPDATE a row"
    
    Case 28567
    sError = "SSCE_M_OPENTABLEFAILED"
    sReason = "Failed to OPEN a table"
    
    Case 28568
    sError = "SSCE_M_CLOSETABLEFAILED"
    sReason = "Failed to CLOSE a table"
    
    Case 28569
    sError = "SSCE_M_INVALIDINDEX"
    sReason = "Failed to Set the Index for seeking on the Row Guid column"
    
    Case 28570
    sError = "SSCE_M_ERRORFETCHINGDATA"
    sReason = "Failed to Get the row data"
    
    Case 28571
    sError = "SSCE_M_ERRORFETCHINGCOLUMNDATA"
    sReason = "Failed to Get the column data for a row"
    
    Case 28572
    sError = "SSCE_M_ERRORSETTINGDATA"
    sReason = "Failed to Set the row data"
    
    Case 28800
    sError = "SSCE_WRN_RDAERRORROWSRETURNED"
    sReason = "One or more error rows returned from RDA Push,see associated _err table"
    
    Case 28801
    sError = "SSCE_WRN_SUBWINSERRORSFIXED"
    sReason = "A data error occurred and was fixed,a subsequent run is required for data"
    
    Case 29000
    sError = "SSCE_M_UNEXPECTEDSTATEREADINGGENINFO"
    sReason = "Unexpected state when reading generation info"
    
    Case 29001
    sError = "SSCE_M_EXPECTEDINSERTSONLY"
    sReason = "Expected inserts only"
    
    Case 29002
    sError = "SSCE_M_IDENTITYSEEDORRANGETOOLARGE"
    sReason = "Identity seed or range is too large"
    
    Case 29003
    sError = "SSCE_M_UNEXPECTEDSTATEPROCESSINGTOMBSTONES"
    sReason = "Unexpected state when processing tombstones"
    
    Case 29004
    sError = "SSCE_M_CANTCOCREATEMSDAINITIALIZE"
    sReason = "Can't cocreate instance MSDAInitialize"
    
    Case 29005
    sError = "SSCE_M_RSCBWITHSAMEEXCHANGEID"
    sReason = "RSCB with same ExchangeID already exists"
    
    Case 29006
    sError = "SSCE_M_SQLRECONCILERFAILED"
    sReason = "SQL Server Reconciler failed"
    
    Case 29007
    sError = "SSCE_M_QUERYCANNOTBETRACKED"
    sReason = "Query cannot be tracked"
    
    Case 29008
    sError = "SSCE_M_PREPAREUPDATECMDFAILED"
    sReason = "PrepareUpdate command failed"
    
    Case 29009
    sError = "SSCE_M_EXPRESSIONWITHOUTALIAS"
    sReason = "Cannot process expression or literal without alias"
    
    Case 29010
    sError = "SSCE_M_NOPRIMARYKEY"
    sReason = "This table does not have a primary key"
    
    Case 29011
    sError = "SSCE_M_ROWVERCOLUMN"
    sReason = "This table contains a timestamp or other row versioning column"
    
    Case 29012
    sError = "SSCE_M_INVALIDPUSHSEQUENCE"
    sReason = "Thrown when internal error during push operation in server agent"
    
    Case 29013
    sError = "SSCE_M_PUSHPARAMSETONVALUE"
    sReason = "Attempt to set a new param value on top of an existing param value in push"
    
    Case 29014
    sError = "SSCE_M_NOTALLPARAMSETFORPUSH"
    sReason = "Not all the parameters were set for the push operation"
    
    Case 29015
    sError = "SSCE_M_PUSHPKCOLCOUNTMISMATCH"
    sReason = "Mismatch in column count between key data supplied and what expected"
    
    Case 29016
    sError = "SSCE_M_VF_MISSINGSERVERCOLUMN"
    sReason = "Verifying schema can not match up a client side column to a server side column"
    
    Case 29017
    sError = "SSCE_M_VF_MISMATCHEDTYPES"
    sReason = "Mismatched OLE DB type information between client columns and server columns"
    
    Case 29018
    sError = "SSCE_M_VF_MISSINGTABLE"
    sReason = "No columns found,Missing tablename or invalid permissions on server"
    
    Case 29020
    sError = "SSCE_M_NOROWSAFFECTED"
    sReason = "A row could not be updated/deleted on the server using RDA"
    
    Case 29021
    sError = "SSCE_M_TOOMANYROWSAFFECTED"
    sReason = "More than one row was affected by the update/delete on the server using RDA"
    
    Case 29022
    sError = "SSCE_M_INCORRECTPROVIDERVERSION"
    sReason = "Incorrect version of SQL Server OLEDB provider"
    
    Case 29023
    sError = "SSCE_M_INCORRECTPROVIDER"
    sReason = "Incorrect provider"
    
    Case 29024
    sError = "SSCE_M_VF_PRIMARYKEYCHANGE"
    sReason = "Push operation could not match client side pk column order to server side pk"
    
    Case 29025
    sError = "SSCE_M_VF_MISSINGSERVERPKCOLUMN"
    sReason = "Push operation: expected primary key column missing on server side"
    
    Case 29026
    sError = "SSCE_M_VF_EXTRASERVERPKCOLUMN"
    sReason = "Push operation: found extra server side primary key column"
    
    Case 29027
    sError = "SSCE_M_MISSINGCOLUMNSCHEMACOLS"
    sReason = "DBSCHEMA_COLUMNS supported by server db provider does not have"
    
    Case 29028
    sError = "SSCE_M_UNSUPPORTEDOLEDBTYPE"
    sReason = "Server code encountered an unsupported OLE DB type reported by server db"
    
    Case 29029
    sError = "SSCE_M_VF_NONLOBIUNKNOWN"
    sReason = "Encountered DBTYPE_IUNKNOWN binding,but DBCOLUMNFLAGS_ISLONG flag was not set"
    
    Case 29030
    sError = "SSCE_M_NOTRACKPKCOLMISSING"
    sReason = "Query cannot be tracked,not all columns in the primary key are included."
    
    Case 29031
    sError = "SSCE_M_NOTRACKCOMPUTEDCOLUMN"
    sReason = "Query cannot be tracked,Attempting to pull a computed column."
    
    Case 29032
    sError = "SSCE_M_NOTRACKMULTITABLE"
    sReason = "Query cannot be tracked,It is a multi-table query."
    
    Case 29033
    sError = "SSCE_M_INVALIDCODEPAGE"
    sReason = "Server or override code page is invalid or not installed on IIS Server"
    
    Case 29500
    sError = "SSCE_M_TRANSPORTREADWRONGSIZE"
    sReason = "Transport read returned incorrect data length"
    
    Case 29501
    sError = "SSCE_M_UNEXPECTEDSTATEWRITINGMESSAGE"
    sReason = "Unexpected state when writing message"
    
    Case 29502
    sError = "SSCE_M_INVALIDCOLUMNNUMBER"
    sReason = "Invalid column number"
    
    Case 29503
    sError = "SSCE_M_CANTFINDCOLUMNBYNAME"
    sReason = "Can not find a column by its name"
    
    Case 29504
    sError = "SSCE_M_NAMESBUFFERTOOSMALL"
    sReason = "Names buffer is too small"
    
    Case 29505
    sError = "SSCE_M_INCORRECTNUMBEROFCOLUMNSPROCESSED"
    sReason = "Incorrect number of columns processed"
    
    Case 29506
    sError = "SSCE_M_UNEXPECTEDSTATEGETTINGCOLUMNS"
    sReason = "Unexpected state when getting columns"
    
    Case 29507
    sError = "SSCE_M_UNEXPECTEDSTATEGETCOLLENGTH"
    sReason = "Unexpected state when getting column length"
    
    Case 29508
    sError = "SSCE_M_UNEXPECTEDSTATEGETCOLDATA"
    sReason = "Unexpected state when getting column data"
    
    Case 29509
    sError = "SSCE_M_UNEXPECTEDSTATESETCOLUMN"
    sReason = "Unexpected state when setting a column"
    
    Case 29510
    sError = "SSCE_M_STREAMNOTEXHAUSTED"
    sReason = "Stream was not exhausted"
    
    Case 29511
    sError = "SSCE_M_GARBLEDMESSAGEFROMSERVER"
    sReason = "Message received from server cannot be decoded"
    
    Case 29512
    sError = "SSCE_M_GARBLEDMESSAGEFROMCLIENT"
    sReason = "Message received from client cannot be decoded"
  
  End Select
sErrorDefine = sError
SSCEErrorDescription = sReason
End Function

